// MainFrm.cpp

#include "stdafx.h"
#include "midixchg.h"
#include "MainFrm.h"

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_UPDATE_COMMAND_UI(IDC_DEVICES, OnUpdateCommands)
	ON_COMMAND(IDC_SEND, OnSend)
	ON_COMMAND(IDC_DEVICES, OnDevices)
	ON_UPDATE_COMMAND_UI(IDC_SEND, OnUpdateCommands)
	ON_UPDATE_COMMAND_UI(IDC_RECEIVE, OnUpdateCommands)
	ON_COMMAND(IDC_RECEIVE, OnReceive)
END_MESSAGE_MAP()

BOOL CMainFrame::m_bSending;
BOOL CMainFrame::m_bReceiving;
BOOL CMainFrame::m_bPostResults;
BOOL CMainFrame::m_bReceivingEvents;
int CMainFrame::m_nPrevNote;
int CMainFrame::m_nNotesDropped;
int CMainFrame::m_nTotalNotes;
DWORD CMainFrame::m_dwStartTime;
DWORD CMainFrame::m_dwStopTime;

CMainFrame::CMainFrame()
{
	Create(0, "MIDI Xchg", WS_OVERLAPPEDWINDOW, rectDefault, 0, MAKEINTRESOURCE(IDR_MAINFRAME));
	m_bSending = FALSE;
	m_bReceiving = FALSE;
}
CMainFrame::~CMainFrame()
{
	delete m_pMidiIn;
	delete m_pMidiOut;
	delete m_pCDeviceDlg;
}
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	InitializeReceiver();
	m_pMidiIn = new MidiIn;
	m_pMidiOut = new MidiOut;
	m_pMidiIn->RegisterMIDIFunction(&CMainFrame::MidiInProc);
	m_pCDeviceDlg = new CDeviceDlg(m_pMidiIn, m_pMidiOut);
	return 0;
}
void CMainFrame::InitializeReceiver()
{
	m_nPrevNote = 0;
	m_nNotesDropped = 0;
	m_nTotalNotes = 0;
	m_bPostResults = FALSE;
	m_bReceivingEvents = FALSE;
	m_bReportedEventStream = FALSE;
	m_nY = 0;
	m_dwStartTime = 0;
	m_dwStopTime = 0;
}
void CMainFrame::OnUpdateCommands(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}
void CALLBACK CMainFrame::MidiInProc(
						HMIDIIN hMidiIn,	
						UINT wMsg,	
						DWORD dwInstance,	
						DWORD dwParam1,	
						DWORD dwParam2	
					   )
{
	// --- extract MIDI message components
	unsigned char status = (unsigned char)(dwParam1 & 0xff);
	switch (status)	{
		case 0xfa:
			// start
			m_nPrevNote = 0;
			m_nNotesDropped = 0;
			m_nTotalNotes = 0;
			m_bPostResults = FALSE;
			m_bReceivingEvents = FALSE;
			m_dwStartTime = timeGetTime();
			break;
		case 0xfc:
			// stop
			m_bReceiving = FALSE;
			m_bPostResults = TRUE;
			m_bReceivingEvents = FALSE;
			m_dwStopTime = timeGetTime();
			break;
		case 0x90:
		// --- note on
		{
			m_bReceivingEvents = TRUE;
			unsigned char note = (((unsigned char)(dwParam1 >> 8)) & 0x7f);
			if (note > m_nPrevNote)
				m_nNotesDropped += note - m_nPrevNote;
			if ((m_nPrevNote = note + 1) == 128)
				m_nPrevNote = 0;
			m_nTotalNotes++;
			break;
		}
		default:
			break;
	}
}
void CMainFrame::OnSend() 
{
	if (m_bSending)	{
		m_bSending = FALSE;
		GetMenu()->ModifyMenu(IDC_SEND, MF_STRING | MF_BYCOMMAND, IDC_SEND, "Send events");
	}
	else	{
		m_bSending = TRUE;
		GetMenu()->ModifyMenu(IDC_SEND, MF_STRING | MF_BYCOMMAND, IDC_SEND, "Stop Sending");
		AfxBeginThread(&CMainFrame::SendEvents, m_pMidiOut);
	}
	DrawMenuBar();
}

UINT CMainFrame::SendEvents(void* pMidiOut)
{
	((MidiOut*)pMidiOut)->StartMessage();
	while (m_bSending)	{
		for (int note = 0; note < nMaxCaptures; note++)	{
			DWORD dwEvent = 0x90 | (note << 8) | (64 << 16);
			((MidiOut*)pMidiOut)->SendEvent(dwEvent);
		}
	}
	((MidiOut*)pMidiOut)->StopMessage();
	return 0;
}
void CMainFrame::OnDevices() 
{
	if (m_pCDeviceDlg->DoModal() == IDOK)	{
		if (!m_pMidiIn->ChangeDevice(m_pCDeviceDlg->m_nInDevice))
			AfxMessageBox("Cannot open MIDI In device", MB_ICONSTOP);
		if (!m_pMidiOut->ChangeDevice(m_pCDeviceDlg->m_nOutDevice))
			AfxMessageBox("Cannot open MIDI Out device", MB_ICONSTOP);
	}
}
void CMainFrame::DisplayMessage(const CString& strMsg)
{
	CDC* pDC = GetDC();
	pDC->TextOut(0, m_nY, strMsg);
	CSize size = pDC->GetTextExtent(strMsg);
	m_nY += size.cy;
}
void CMainFrame::OnReceive() 
{
	InitializeReceiver();
	// --- clear the screen
	InvalidateRect(0);
	UpdateWindow();
	m_bReceiving = TRUE;
	DisplayMessage("Waiting for events...");
}
void CMainFrame::PostResults()
{
	m_bPostResults = FALSE;
	CString strMsg;
	strMsg.Format("Total events received: %d", m_nTotalNotes);
	DisplayMessage(strMsg);
	strMsg.Format("Total events dropped:  %d", m_nNotesDropped);
	DisplayMessage(strMsg);
	DWORD dwTime = m_dwStopTime - m_dwStartTime;
	strMsg.Format("Elapsed time (ms):     %ld", dwTime);
	DisplayMessage(strMsg);
	DWORD dwEvSec = m_nTotalNotes * 1000 / dwTime;
	strMsg.Format("Events/second:         %ld", dwEvSec);
	DisplayMessage(strMsg);
	strMsg.Format("Approximate baud rate: %ld", dwEvSec * 24);
	DisplayMessage(strMsg);
}
void CMainFrame::ReportEventStream()
{
	m_bReportedEventStream = TRUE;
	DisplayMessage("Receiving events...");
}
